//==============================================
//  setWholeBarRestsVisible v0.1
//
//  Copyright (C)2015 Oliver Voelzel 
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================
import QtQuick 2.0
import MuseScore 1.0

MuseScore {
      menuPath: "Plugins.WholeBarRests.SetVisible"
      description: "This plugin set all whole bar rests visible"
      property bool processAll: false


      function processPart(cursor,endTick,startTrack,endTrack) {
            if(processAll) {
                  // we need to reset track first, otherwise
                  // rewind(0) doesn't work correctly

                  cursor.staffIdx = startTrack / 4;
                  cursor.voice = 0; 
                  cursor.rewind(0);
            } else {
                  cursor.rewind(1);

                  cursor.staffIdx = startTrack / 4;
                  cursor.voice = 0;
            }

            var segment = cursor.segment;

            // we use the cursor to know measure boundaries
 
            cursor.nextMeasure();

            var curMeasureArray = new Array();

            // we use a segment, because the cursor always proceeds to 
	    // the next element in the given track and we don't know 
	    // in which track the element is.
            var inLastMeasure=false;
            while(segment && (processAll || segment.tick < endTick)) {
                  // check if still inside same measure
                  if(!inLastMeasure && !(segment.tick < cursor.tick)) {
                        // new measure
                        curMeasureArray = new Array();
                        //keySig = cursor.keySignature;
                        if(!cursor.nextMeasure()) {
                              inLastMeasure=true;
                        }
                  }

                  for(var track=startTrack; track<endTrack; track++) {
                        if(segment.elementAt(track) && segment.elementAt(track).type == Element.REST) {
                              
                              // process graceNotes if present
                              if(segment.elementAt(track).isFullMeasure) {
                                    segment.elementAt(track).visible=true
                                    
                                    
                                    }
                              }


                        }
                  
                  segment=segment.next;
            }
      }

      function removeAcc() {
            console.log("start set whole bar rests visible");

            //curScore.startCmd();

             if (typeof curScore === 'undefined' || curScore == null) {
                   console.log("error: no score!");	     
                   Qt.quit();
             }

            // find selection
            var startStaff;
            var endStaff;
            var endTick;
            
            var cursor = curScore.newCursor();
            cursor.rewind(1);
            if(!cursor.segment) {
                  // no selection
                  console.log("no selection: processing whole score");
                  processAll = true;
                  startStaff = 0;
                  endStaff = curScore.nstaves;
            } else {
                  startStaff = cursor.staffIdx;
                  cursor.rewind(2);
                  endStaff = cursor.staffIdx+1;
                  endTick = cursor.tick;
                  if(endTick == 0) {
                        // selection includes end of score
                        // calculate tick from last score segment
                        endTick = curScore.lastSegment.tick + 1;
                  }
                  cursor.rewind(1);
                  console.log("Selection is: Staves("+startStaff+"-"+endStaff+") Ticks("+cursor.tick+"-"+endTick+")");
            }      

            console.log("ProcessAll is "+processAll);

            // go through all staves of a part simultaneously
            // find staves that belong to the same part

            var curStartStaff = startStaff;

            while(curStartStaff < endStaff) {
                  // to remove we need to parse staff by staff
                  var curEndStaff = curStartStaff+1;

                  // do the work
                  processPart(cursor,endTick,curStartStaff*4,curEndStaff*4);

                  // next part
                  curStartStaff = curEndStaff;
            }

            //curScore.endCmd();
            //curScore.doLayout();

            console.log("end set whole bar rests visible");
            Qt.quit();
      }

      onRun: { 
            removeAcc();
      }
}
